#ifndef _COMMON_H_
#define _COMMON_H_
#include <stm32f0xx.h>
#include "stdbool.h"

#define FCPU           48000000UL
//#define F_CPU         8000000UL
#define TimerTick	FCPU/1000-1

#ifndef TRUE_INLINE
#define TRUE_INLINE  __attribute__((always_inline)) static inline
#endif

#define pin_toggle(gpioport, gpios)  do{  \
        register uint32_t __port = gpioport->ODR;  \
        gpioport->BSRR = ((__port & (gpios)) << 16) | (~__port & (gpios));}while(0)
#define pin_set(gpioport, gpios)  do{gpioport->BSRR = gpios;}while(0)
#define pin_clear(gpioport, gpios) do{gpioport->BSRR = ((gpios) << 16);}while(0)
#define pin_read(gpioport, gpios) (gpioport->IDR & (gpios) ? 1 : 0)
#define pin_write(gpioport, gpios)  do{gpioport->ODR = gpios;}while(0)

//############################################################################//
typedef unsigned int word;
#define bit(b) (1UL << (b))
typedef bool boolean;
typedef uint8_t byte;
#define I2CWAIT  (30)
#define lowByte(w)  ((uint8_t) ((w) & 0xff))  //(uint8_t)(address &0x00FF);
#define highByte(w) ((uint8_t) ((w) >> 8))    //(address>>8);

#define min(a,b)                ((a)<(b)?(a):(b))
#define max(a,b)                ((a)>(b)?(a):(b))
#define abs(x)                  ((x)>0?(x):-(x))
#define constrain(amt,low,high) ((amt)<(low)?(low):((amt)>(high)?(high):(amt)))
#define round(x)                ((x)>=0?(long)((x)+0.5):(long)((x)-0.5))
#define radians(deg)            ((deg)*DEG_TO_RAD)
#define degrees(rad)            ((rad)*RAD_TO_DEG)
#define sq(x)                   ((x)*(x))

#define bitRead(value, bit)     (((value) >> (bit)) & 0x01)
#define bitSet(value, bit)      ((value) |= (1UL << (bit)))
#define bitClear(value, bit)    ((value) &= ~(1UL << (bit)))
#define bitToggle(value, bit)   ((value) ^= (1UL << (bit)))
#define bitWrite(value, bit, bitvalue) ((bitvalue) ? bitSet(value, bit) : bitClear(value, bit))

void gpio_init(void);
void systick_init(void);
void rcc_sysclockinit(void);
void zi2c_init (void);
uint8_t zi2c_write(uint8_t addr, uint8_t *data, uint8_t nbytes);
uint8_t zi2c_read(uint8_t addr, uint8_t *data, uint8_t nbytes);

uint8_t bcd2dec(uint8_t data);
uint8_t dec2bcddiv(uint8_t data);
uint8_t dec2bcd(uint8_t n);

uint8_t dec2bcd4(uint8_t n);
uint8_t bcd2dec4(uint8_t n);
void _delay_ms(uint32_t dtime);

#define _bitset(bits) ((\
  (bits%010)|\
  (bits/010%010)<<1|\
  (bits/0100%010)<<2|\
  (bits/01000%010)<<3|\
  (bits/010000%010)<<4|\
  (bits/0100000%010)<<5|\
  (bits/01000000%010)<<6|\
  (bits/010000000%010)<<7))
#define bin(bits) _bitset(0##bits) //  u8 a = bin(01100011);

#endif // _COMMON_H_